##############################################################################
####  CopyZClassProduct.py
####  Author: Michael Eichner
##############################################################################


from I_CopyZClassProduct import CopyZClassProduct
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Acquisition import Implicit
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from Globals import DTMLFile
from ZClasses import ZClass,Method, Basic, Property, ZClassOwner
from ZClasses.ZClass import ZClass
from App.Product import Product
from GetOldZClass import GetOldZClass
from WriteZClass import WriteZClass
from string import split,join


class CopyZClassProduct (Implicit, Persistent, RoleManager, Item):
			"""
			CopyZClass Product Class
			"""
			
			__implements__ = CopyZClassProduct
			
			meta_type = '[ME] CopyZClass'
			
			_showWelcome = DTMLFile('dtml/showWelcome', globals())
			_editSetup = DTMLFile('dtml/editSetup', globals())
			_editSetup_Product = DTMLFile('dtml/editSetup_Product', globals())
			_editSetup_ZClass = DTMLFile('dtml/editSetup_ZClass', globals())
			_editSetupDestination = DTMLFile('dtml/editSetupDestination', globals())
			_editSetupDestination_error = DTMLFile('dtml/editSetupDestination_error', globals())			
			_SetupBases = DTMLFile('dtml/SetupBases', globals())
			
			manage_options = ( {'label':'Welcome','action':'showWelcome'},
										 	 	 {'label':'Setup: Source','action':'editSetup'},
												 {'label':'Setup: Destination','action':'editSetupDestination'},
												 {'label':'Copy','action':'create_ZClass'},
												 {'label':'Replace ZClass','action':'switch_ZClass'},
										 	 	 {'label':'Source ZClass','action':'showOldZClassSummary'},
												 {'label':'New ZClass','action':'showNewZClassSummary'},
												 #{'label':'Internal Debug Info','action':'showInternalDebugInfo'},												 
											 )
			
			security = ClassSecurityInfo()
			
			def __init__(self,id):
					self.id = id
					self.__oldZClassPath = ""
					self.__newZClassPath = ""
					self.__ProductPath = ""
					self.__subclassableClasses = ""
					self.__basematrix = ""
					self._p_changed = 1
					
					
			def showWelcome(self):
					""" Print Welcome Screen """
					return self._showWelcome(self.REQUEST)
					
					
			def editSetup(self):
					""" Print Setup Screen """
					if self.get_oldZClassPath()=="":
						 return self.editSetup_Product()
					else:
						 return self.editSetup_ZClass(0)
					
					
			def editSetup_Product(self):
					""" Print Setup Screen to choose product """
					self.__internal_products = []
					self.__internal_products = self.get_internalProducts()
					self.REQUEST.set('internal_products',self.__internal_products)
					return self._editSetup_Product(self.REQUEST,management_view="Setup: Source")
			
			
			def editSetup_ZClass(self,par_saved):
					""" 
					Print Setup Screen to choose ZClass
					par_saved decides whether a message about a successful previous "save"-action should appear. 
					  0 : no
						1 : yes
					"""
					self.__zclassesofprod = []
					self.__zclassesofprod = self.get_ZClassesOfProduct(self.get_ProductName())
					self.REQUEST.set('zclassesofprod',self.__zclassesofprod)
					if par_saved==0:
						 return self._editSetup_ZClass(self.REQUEST,management_view="Setup: Source")
					else:
						 return self._editSetup_ZClass(self.REQUEST,manage_tabs_message="Setup saved",management_view="Setup: Source")

			
			def saveSetup(self):
					""" Save Setup """
					#self.__oldZClassPath = self.REQUEST['s_oldZClassPath']
					self.__oldZClassPath = "/Control_Panel/Products/" + self.REQUEST['sel_ProductName'] + "/" + self.REQUEST['sel_zclasstocopy']
					self.__ProductPath = self.get_ProductPath()
					#self.__subclassableClasses = self.get_subclassableClasses()
					#self.__basematrix = self.init_basematrix()						 
					self._p_changed = 1
					if self.REQUEST['sel_zclasstocopy']=="pleasechoose":
						 return self.editSetup_ZClass(0)
					else:						 
						 self.__subclassableClasses = self.get_subclassableClasses()
						 self.__basematrix = self.init_basematrix()
						 self._p_changed = 1
						 return self.editSetup_ZClass(1)
											
					
			def SetupBases(self):
					""" 
					Provide _SetupBases with information about the base classes of the ZClass 
					that is to be copied and all available base classes.
					_SetupBases called to display the setup screen to choose additional base classes
					"""					
					self.REQUEST.set('basematrix',self.__basematrix)
					return self._SetupBases(self.REQUEST,management_view="Setup: Source")
					
					
			def saveSetupBases(self):
					""" Save setup of base classes """
					if self.REQUEST['saveSetupBases']=='  >>  ':
						 try:
						 		 if self.REQUEST['unselectedbases']:
						 		 		for alauf in self.REQUEST['unselectedbases']:
												for ilauf in self.__basematrix:
														if alauf==ilauf['basepath']: ilauf['status']='s'
						 except:
	 					 		 pass
					elif self.REQUEST['saveSetupBases']=='  <<  ':
						 try:
						 		 if self.REQUEST['selectedbases']:
						 		 		for alauf in self.REQUEST['selectedbases']:
												for ilauf in self.__basematrix:
														if alauf==ilauf['basepath']: ilauf['status']='u'
						 except:
	 					 		 pass
					self.REQUEST.set('basematrix',self.__basematrix)
					self._p_changed = 1
					return self._SetupBases(self.REQUEST,manage_tabs_message="Setup saved",management_view="Setup: Source") 
			
			
			def editSetupDestination(self,par_saved=0):
					""" Setup of destination """
					#if self.get_oldZClassPath()=="":
					#	 return self._editSetupDestination_error(self.REQUEST,management_view="Setup: Destination")
					#else:
					self.__internal_products = []
					self.__internal_products = self.get_internalProducts()
					self.REQUEST.set('internal_products',self.__internal_products)
					if (par_saved==0):
						 return self._editSetupDestination(self.REQUEST,management_view="Setup: Destination")
					else:
						 return self._editSetupDestination(self.REQUEST,manage_tabs_message="Setup of destination ZClass saved",management_view="Setup: Destination")
					
					
			def saveSetupDestination(self):
					""" Save path of destination ZClass """
					self.__newZClassPath = "/Control_Panel/Products/" + self.REQUEST['sel_newProductName'] + "/" + self.REQUEST['dest_id']
					return self.editSetupDestination(1)
			
			
			def create_ZClass(self):
					"""
					Show view to go on after new ZClass is created
					"""
					me__tmp = ""
					try:
							self._create_ZClass()														
							me__tmp = me__tmp + "<h2><center>New ZClass creation successful</center></h2>"
					except:
							me__tmp = me__tmp + "<h2><center>New ZClass already exists</center></h2>"
					me__tmp = me__tmp + "<center><a href=\"transferData\">Next >>></a></center>"
					return self.manage_page_header()+self.manage_tabs()+ \
								 me__tmp + \
								 self.manage_page_footer()
			
			
			def transferData(self):
					"""
					Generate and fill a new ZClass with data of the old class
					"""					
					me__tmp = ""
					me__tmp = me__tmp + "<pre>\n"

					old_instance = GetOldZClass(self,self.get_oldZClassPath())
					new_instance = WriteZClass(self,self.get_newZClassPath())
					new_instance.czc_set_basic_iconpath(old_instance.czc_get_basic_iconpath())
					new_instance.czc_set_basic_iconimage(old_instance.czc_get_basic_iconimage())
					new_instance.czc_set_DefinedPermissions(old_instance.czc_get_DefinedPermissions())
					new_instance.czc_set_PermissionMapping(old_instance.czc_get_PermissionMapping())
					new_instance.czc_copy_methods(old_instance.czc_get_oldZClassObj_methods(),old_instance.czc_get_methods(),old_instance.czc_get_oldZClassPath_methods())
					new_instance.czc_set_views(old_instance.czc_get_views())
					for psids in old_instance.czc_get_ps_IdsTitle():
							new_instance.czc_create_ps(psids['id'],psids['title'])
							new_instance.czc_set_ps_Items(psids['id'],old_instance.czc_get_ps_Items(psids['id']))
							new_instance.czc_set_ps_Permissions(psids['id'],old_instance.czc_get_ps_Permissions(psids['id']))					
										
					me__tmp = me__tmp + "<h2><center>ZClass successful copied !!</center></h2>"
					me__tmp = me__tmp + "</pre>"		
					return self.manage_page_header()+self.manage_tabs()+ \
								 me__tmp + \
								 self.manage_page_footer()
			
			
			def switch_ZClass(self):
					"""
					Show view to replace the old ZClass with the new one
					"""
					me__tmp = ""					
					me__tmp = me__tmp + "<h2><center>Replace source (old) ZClass with new ZClass </center></h2>"
					me__tmp = me__tmp + "<center><a href=\"do_switch_ZClass\">Next >>></a></center>"
					return self.manage_page_header()+self.manage_tabs()+ \
								 me__tmp + \
								 self.manage_page_footer()
			
			
			def do_switch_ZClass(self):
					"""
					Replace the old ZClass with the new one
					"""
					me__tmp = ""					
					
					prodobj = self.unrestrictedTraverse(self.__ProductPath)					
					old_instance = GetOldZClass(self,self.get_oldZClassPath())
					new_instance = WriteZClass(self,self.get_newZClassPath())
					tmp_instance = GetOldZClass(self,self.get_newZClassPath())					
					tmpoldid = old_instance.czc_getId()
					prodobj._delObject(tmpoldid)
					prodobj.manage_renameObject(tmp_instance.czc_getId(),tmpoldid)
					
					me__tmp = me__tmp + "<h2><center>Source (old) ZClass replaced successfully !!!</center></h2>"
					return self.manage_page_header()+self.manage_tabs()+ \
								 me__tmp + \
								 self.manage_page_footer()
			
			
			def showOldZClassSummary(self):
					"""
					Call showZClassSummary to show ZClass that is to be copied
					"""
					try:
							return self.showZClassSummary(self.get_oldZClassPath())
					except:
							me__tmp = ""
							me__tmp = me__tmp + "<h2><center>Error: Source ZClass not found or specified. Source ZClass can be set using the Setup-View.</center></h2>"
							return self.manage_page_header()+self.manage_tabs()+ \
								     me__tmp + \
								     self.manage_page_footer()
					
					
			def showNewZClassSummary(self):
					"""
					Call showZClassSummary to show new ZClass 
					"""
					try:
							return self.showZClassSummary(self.get_newZClassPath())
					except:
							me__tmp = ""
							me__tmp = me__tmp + "<h2><center>Error: New ZClass cannot be found. </center></h2>"
							return self.manage_page_header()+self.manage_tabs()+ \
								     me__tmp + \
								     self.manage_page_footer()
			
			
			def showZClassSummary(self,par_path):
					"""
					Show information about a ZClass specified by par_path 										 
					"""
					me__tmp = ""
					me_instance = GetOldZClass(self,par_path)
					me__tmp = me__tmp + "<pre>"
					me__tmp = me__tmp + "******************************************************************************\n"
					me__tmp = me__tmp + "ZClass:   " + me_instance.czc_getId() + "\n"
					me__tmp = me__tmp + "******************************************************************************\n"					
					me__tmp = me__tmp + ">>>>>> Methods <<<<<<\n"
					me__tmp = me__tmp + str(me_instance.czc_get_methods()) + "\n"
					me__tmp = me__tmp + "\n\n"
					
					me__tmp = me__tmp + ">>>>>> Basic <<<<<<\n"
					me__tmp = me__tmp + "Bases:  " + str(me_instance.czc_get_basic_bases()) + "\n"
					me__tmp = me__tmp + "Title:  " + str(me_instance.czc_get_basic_title()) + "\n"					
					me__tmp = me__tmp + "MetaType:  " + str(me_instance.czc_get_basic_metatype()) + "\n"
					me__tmp = me__tmp + "Class ID:  " + str(me_instance.czc_get_basic_classid()) + "\n"
					me__tmp = me__tmp + "Icon Path:  " + str(me_instance.czc_get_basic_iconpath()) + "\n"					
					me__tmp = me__tmp + "\n\n"
					
					me__tmp = me__tmp + ">>>>>> Views <<<<<<\n"
					me__tmp = me__tmp + str(me_instance.czc_get_views()) + "\n"
					for me__lauf in me_instance.czc_get_views():
							me__tmp = me__tmp + str(me__lauf) + "\n"
					me__tmp = me__tmp + "\n\n"
					
					me__tmp = me__tmp + ">>>>>> Property Sheets <<<<<<\n"
					for me__lauf in me_instance.czc_get_ps_Ids():
							me__tmp = me__tmp + "-----   " + str(me__lauf) + " (Properties)  -----" + "\n"
							for me__lauf2 in me_instance.czc_get_ps_Items(me__lauf):
									me__tmp = me__tmp + str(me__lauf2) + "\n"
							me__tmp = me__tmp + "-----   " + str(me__lauf) + " (Permissions)  -----" + "\n"
							for me__lauf2 in me_instance.czc_get_ps_Permissions(me__lauf):
									me__tmp = me__tmp + str(me__lauf2) + "\n"						
							me__tmp = me__tmp + "\n"
					me__tmp = me__tmp + "\n"								
					
					me__tmp = me__tmp + ">>>>>> Permissions <<<<<<\n"
					me__tmp = me__tmp + "-----   Defined   -----\n"
					me__tmp = me__tmp + str(me_instance.czc_get_DefinedPermissions()) + "\n"
					me__tmp = me__tmp + "-----   Inherited   -----\n"
					for me__lauf in me_instance.czc_get_InheritedPermissions(): 
							me__tmp = me__tmp + str(me__lauf) + "\n"			
					me__tmp = me__tmp + "\n\n\n"
					
					me__tmp = me__tmp + ">>>>>> Define Permissions <<<<<<\n"
					for me__lauf in me_instance.czc_get_PermissionMapping(): 
							me__tmp = me__tmp + str(me__lauf) + "\n"
					
					me__tmp = me__tmp + "\n\n"
					
					me__tmp = me__tmp + "</pre>"
					return self.manage_page_header()+self.manage_tabs()+ \
								 me__tmp + \
								 self.manage_page_footer()

			
			def getInternalDebugInfo(self):
					"""
					Set the specific debug information 
					shown by showInternalDebugInfo (temp. product view)
					"""					
					me__do = ""											
					me__do = me__do + "<pre>"
					me__do = me__do + "==================== ZClass erzeugen ===========================\n"
					me__do = me__do + "foundobj = self.unrestrictedTraverse('/Control_Panel/Products/BRK_Bergwacht/ZC__AG_Global/propertysheets/common/AG_Global')\n"
					foundobj = self.unrestrictedTraverse('/Control_Panel/Products/BRK_Bergwacht/ZC__AG_Global/propertysheets/common/AG_Global')					
					me__do = me__do + "Id:  " + str(foundobj.getId())
					me__do = me__do + "\n"
					#me__do = me__do + "getProperty: " + str(foundobj.propertyMap().getProperty('BRK_BW_Funktion')) + "\n"
					me__do = me__do + "getProperty: " + str(foundobj.getProperty('BRK_BW_Funktion')) + "\n"
					#me__do = me__do + "xxobjectIds:  " + str(foundobj.objectIds())
					#me__do = me__do + "\n"
					#objstofilter =  foundobj.objectIds()
					#me__count = 0
					#for objtocheck in objstofilter:
					#		me__count = me__count + 1
					#		tmp_obj = self.unrestrictedTraverse('/Control_Panel/Products/'+objtocheck)
					#		if isinstance(tmp_obj,Product):
					#			 if (str(tmp_obj.icon)=="p_/Product_icon"):
					#			 		me__do = me__do + "iiobjectIds:  " + str(objtocheck) + "\n"
					#			 else:										
					#			 		me__do = me__do + "externes Produkt:  " + str(objtocheck) + "\n"
					#		
					#me__do = me__do + "me__count:  " + str(me__count) + "\n"
					#Z=ZClass(id, title, bases, zope_object=zope_object)
					###Z=ZClass.ZClass("autoclassid", "autoclasstitle", "", zope_object=0)
					#Z._zclass_.meta_type=meta_type
					###Z._zclass_.meta_type="autoclassmetatype"
					#self._setObject(id, Z)
					###foundobj._setObject("autoclassid", Z)
					me__do = me__do + "\n\n"
					me__do = me__do + "==================== classnames ===========================\n"
					#me__do = me__do + "manage_subclassableClassNames():  " + str(foundobj.manage_subclassableClassNames())
					me__do = me__do + "manage_subclassableClassNames():  " + str(self.get_subclassableClasses())
					me__do = me__do + "\n\n"
					me__do = me__do + "==================== ProductPath ===========================\n"
					me__do = me__do + self.get_ProductPath() + "\n"
					me__do = me__do + "\n\n"
					me__do = me__do + "==================== dicttuple ===========================\n"		
					maptmp1 = {}
					maptmp2 = {}
					maptmp1['aa'] = 'ss'
					maptmp1['bb'] = 'ss'
					maptmp2['aa'] = 'ss'
					maptmp2['bb'] = 'ss'
					dicttuple = ()
					dicttuple1 = ()
					dicttuple1 = (maptmp1,)
					dicttuple2 = ()
					dicttuple2 = (maptmp2,)
					dicttuple = dicttuple+dicttuple1
					dicttuple = dicttuple+dicttuple2			
					me__do = me__do + str(dicttuple) + "\n"										
					me__do = me__do + "\n\n"					 											
					me__do = me__do + "</pre>"
					return me__do

					
			def showInternalDebugInfo(self):
					""" 
					Show Internal Debug Information
					View should be disabled in final release			
					"""
					return self.manage_page_header()+self.manage_tabs()+ \
								 self.getInternalDebugInfo() + \
								 self.manage_page_footer()
								 
								 
			def get_oldZClassPath(self):
					""" Return __oldZClassPath """
					return self.__oldZClassPath
					
					
			def get_newZClassPath(self):
					""" Return path of the new ZClass """
					#me_instance = GetOldZClass(self,self.get_oldZClassPath())
					#newid = "czc__" + me_instance.czc_getId()
					#return self.__ProductPath + "/" + newid
					return self.__newZClassPath												
					
					
			def get_ProductPath(self):
					""" 
					Generate path to product of specified class
					**IMPORTANT** At the moment it's expected that the ZClass is right in the product folder
					"""
					teile = split(self.get_oldZClassPath(),'/')					
					teile.pop()
					teile = join(teile,'/')
					return str(teile)
					
					
			def get_ProductName(self):
					"""
					Get name of selected Product
					"""
					if self.get_oldZClassPath()!="":
						 teile = split(self.get_oldZClassPath(),'/')
						 teile.pop()					
						 return teile.pop()
					else:
						 return ""
			
			
			def get_ZClassName(self):
					"""
					Get name of selected ZClass
					"""
					teile = split(self.get_oldZClassPath(),'/')					
					return teile.pop()
			
			
			def get_newProductName(self):
					"""
					Get name of the destination Product
					"""
					if self.get_newZClassPath()!="":
						 teile = split(self.get_newZClassPath(),'/')
						 teile.pop()					
						 return teile.pop()
					else:
						 return ""
			
			
			def get_newProductPath(self):
					""" Return path to product of destination ZClass """
					return "/Control_Panel/Products/"+self.get_newProductName()
			
			
			def get_newZClassName(self):
					"""
					Get name of the destination ZClass
					"""
					teile = split(self.get_newZClassPath(),'/')					
					return teile.pop()
										
					
			def get_subclassableClasses(self):
					"""
					Get a list of all subclassable class names
					"""
					tmpobj = self.unrestrictedTraverse(self.__ProductPath)
					return tmpobj.manage_subclassableClassNames()
					
			
			def get_internalProducts(self):
					"""
					Get a list of all internal products (exclusion of all external installed products).
					Only internal products can contain ZClasses.
					Used by: Setup view to select desired product
					Note: installed product recognized by the iconname at the moment. Therefore this
					solution has to be handled with caution. Any more secure solution appreciated.
					!! Compatibility: Zope 2.4.3 !!
					!! Zope Version 2.5.x and higher not checked yet !! 
					"""
					internalproducts = []
					productobj = self.unrestrictedTraverse('/Control_Panel/Products')
					objstofilter =  productobj.objectIds()
					for objtocheck in objstofilter:
							tmp_obj = self.unrestrictedTraverse('/Control_Panel/Products/'+objtocheck)
							if isinstance(tmp_obj,Product):
								 if (str(tmp_obj.icon)=="p_/Product_icon"):
										internalproducts.append(str(objtocheck))
							
					return internalproducts
					
					
			def get_ZClassesOfProduct(self,par_prod):
					"""
					Get a list of all ZClasses in the specified product "par_prod"
					Used by: Setup view to select desired ZClass
					"""
					zclasslist = []
					productobj = self.unrestrictedTraverse('/Control_Panel/Products/'+par_prod)
					objstofilter =  productobj.objectIds()
					for objtocheck in objstofilter:
							tmp_obj = self.unrestrictedTraverse('/Control_Panel/Products/'+par_prod+"/"+objtocheck)
							if isinstance(tmp_obj,ZClass):
								 zclasslist.append(str(objtocheck))
							
					return zclasslist
			
					
			def init_basematrix(self):
					"""
					Generate and initialize a matrix (list with dictionaries) that contains
					information about the subclassable base classes and set the status
					whether a base clase has already been chosen by the ZClass that is to be copied
					"""
					basematrix = []
					me_instance = GetOldZClass(self,self.get_oldZClassPath())
					for me__lauf in self.get_subclassableClasses():
							maptmp = {}
							teile = split(me__lauf[1],': ')
							maptmp['basepath'] = me__lauf[0]
							maptmp['product'] = teile[0]
							maptmp['zclass'] = teile[1]
							maptmp['paz'] = me__lauf[1]
							maptmp['status'] = 'u'														
							basematrix.append(maptmp)					
					for me__lauf in basematrix:
							for me__lauf2 in me_instance.czc_get_basic_bases():
									if me__lauf['zclass']==me__lauf2:
										 me__lauf['status']='s'
					return basematrix
					
					
			def _create_ZClass(self):
					"""
					Create a new empty ZClass using the base clases specified by the "Setup"-view
					New ID is built using the old ID combined with the prefix "czc__"
					"""
					me_instance = GetOldZClass(self,self.get_oldZClassPath())
					methodsobjlist = me_instance.czc_get_oldZClassObj_methods()
					methodslist = me_instance.czc_get_methods()
					methodspath = me_instance.czc_get_oldZClassPath_methods()					
					#newid = "czc__" + me_instance.czc_getId()
					newid = self.get_newZClassName()
					prodobj = self.unrestrictedTraverse(self.get_newProductPath())
					prodobj.manage_addZClass(newid,me_instance.czc_get_basic_title(),self.get_selectedBasesList(),meta_type=me_instance.czc_get_basic_metatype(),zope_object=1)
					me_new_instance = GetOldZClass(self,self.get_newProductPath()+'/'+newid)
					#me_new_instance = GetOldZClass(self,self.__ProductPath+'/'+newid)
					for lauf in methodslist:
							tmp_obj = self.unrestrictedTraverse(methodspath+'/'+lauf)
							if isinstance(tmp_obj, ZClass): 
								 self._create_containedZClass(methodspath+'/'+lauf,me_new_instance.czc_get_oldZClassPath_methods())
							else:
								 pass	
					
					#tmpmethods = []
					#tmpmethods.append(me_instance.czc_getId()+'_add')
					#tmpmethods.append(me_instance.czc_getId()+'_addForm')
					#tmpmethods.append(me_instance.czc_getId()+'_add_permission')
					#tmpmethods.append(me_instance.czc_getId()+'_factory')
					#tmp_cp = prodobj.manage_copyObjects(tmpmethods)
					#prodobj.manage_pasteObjects(tmp_cp)
					#prodobj.manage_renameObject('czc_'+me_instance.czc_getId()+'_add','czc_'+me_instance.czc_getId()+'_add')
					#prodobj.manage_renameObject('czc_'+me_instance.czc_getId()+'_addForm','czc_'+me_instance.czc_getId()+'_addForm')
					#prodobj.manage_renameObject('czc_'+me_instance.czc_getId()+'_add_permission','czc_'+me_instance.czc_getId()+'_add_permission')
					#prodobj.manage_renameObject('czc_'+me_instance.czc_getId()+'_factory','czc_'+me_instance.czc_getId()+'_factory')															
										
										
			def _create_containedZClass(self,par_path,par_container):
					"""
					Create new empty sub-ZClass(es). Necessary because the methods-view can contain a ZClass itself. 
					A recursive creation of the sub-ZClass(es) is mandatory.
					"""
					me_instance = GetOldZClass(self,par_path)
					methodsobjlist = me_instance.czc_get_oldZClassObj_methods()
					methodslist = me_instance.czc_get_methods()
					methodspath = me_instance.czc_get_oldZClassPath_methods() 
					#newid = "czc__" + me_instance.czc_getId()
					newid = me_instance.czc_getId()
					internalbasematrix = []
					for me__lauf in self.get_subclassableClasses():
							maptmp = {}
							teile = split(me__lauf[1],': ')
							maptmp['basepath'] = me__lauf[0]
							maptmp['product'] = teile[0]
							maptmp['zclass'] = teile[1]
							maptmp['paz'] = me__lauf[1]
							maptmp['status'] = 'u'														
							internalbasematrix.append(maptmp)					
					for me__lauf in internalbasematrix:
							for me__lauf2 in me_instance.czc_get_basic_bases():
									if me__lauf['zclass']==me__lauf2:
										 me__lauf['status']='s'
					selectedBasesList = []															
					for me__lauf in internalbasematrix:
							if me__lauf['status']=='s': selectedBasesList.append(me__lauf['basepath'])										 
					prodobj = self.unrestrictedTraverse(par_container)
					prodobj.manage_addZClass(newid,me_instance.czc_get_basic_title(),selectedBasesList,meta_type=me_instance.czc_get_basic_metatype(),zope_object=1)
					me_new_instance = GetOldZClass(self,par_container+'/'+newid)
					for lauf in methodslist:
							tmp_obj = self.unrestrictedTraverse(methodspath+'/'+lauf)
							if isinstance(tmp_obj, ZClass): 
								 self._create_containedZClass(methodspath+'/'+lauf,me_new_instance.czc_get_oldZClassPath_methods())
							else:
								 pass							
										
										
			def get_selectedBasesList(self):
					"""
					Get selected bases of the basematrix
					"""
					selectedBasesList = []															
					for me__lauf in self.__basematrix:
							if me__lauf['status']=='s': selectedBasesList.append(me__lauf['basepath'])							
					return selectedBasesList
					 
					
InitializeClass(CopyZClassProduct)

