##############################################################################
####  WriteZClass.py
####  Author: Michael Eichner
##############################################################################

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Acquisition import Implicit
from Globals import Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from Globals import DTMLFile
from ZClasses.Property import ZCommonSheet
from ZClasses.ZClass import ZClass
from GetOldZClass import GetOldZClass
from CopyZClassProduct import CopyZClassProduct

class WriteZClass (Implicit, Persistent, RoleManager, Item):
			"""
			Fill the specified ZClass with given information. 
			Methods can be inserted, permissions can be set and propertysheets can be created
			"""
			def __init__(self,par_self,par_newZClassPath): 
					self.__extself = par_self
					self.REQUEST = par_self.REQUEST
					self.__newZClassPath = par_newZClassPath
					self.__newZClassPath_methods = par_newZClassPath+'/propertysheets/methods'
					self.__newZClassPath_basic = par_newZClassPath+'/propertysheets/basic'
					self.__newZClassPath_views = par_newZClassPath+'/propertysheets/views'
					self.__newZClassPath_propertysheets = par_newZClassPath+'/propertysheets/common'
					self.__newZClassObj = par_self.unrestrictedTraverse(par_newZClassPath)
					self.__newZClassObj_methods = par_self.unrestrictedTraverse(par_newZClassPath+'/propertysheets/methods')
					self.__newZClassObj_basic = par_self.unrestrictedTraverse(par_newZClassPath+'/propertysheets/basic')
					self.__newZClassObj_views = par_self.unrestrictedTraverse(par_newZClassPath+'/propertysheets/views')
					self.__newZClassObj_propertysheets = par_self.unrestrictedTraverse(par_newZClassPath+'/propertysheets/common')
			
			
			def czc_set_basic_iconpath(self,par_iconpath):
					"""
					Get iconpath (view: basic)
					"""
					return self.__newZClassObj_basic.manage_edit(icon=par_iconpath)
					
					
			def czc_set_basic_iconimage(self,par_iconimage):
					"""
					Set iconimage attribute (view: basic)
					"""
					return self.__newZClassObj.setClassAttr('ziconImage',par_iconimage)
					
					
			def czc_copy_methods(self,par_oldZClassObj_methods,par_methods,par_oldZClassPath_methods):
					"""
					Copy methods given by par_methods of the old ZClass (view: methods)  
					"""				
					for lauf in par_methods:
							tmp_obj = self.__extself.unrestrictedTraverse(par_oldZClassPath_methods+'/'+lauf)
							if isinstance(tmp_obj, ZClass):
								 self.czc_fill_subzclass(par_oldZClassPath_methods+'/'+lauf,self.__newZClassPath_methods+'/'+lauf)
							else:					
								 tmpmethods = []
								 tmpmethods.append(lauf)
								 tmp_cp = par_oldZClassObj_methods.manage_copyObjects(tmpmethods)
								 self.__newZClassObj_methods.manage_pasteObjects(tmp_cp)							
					
					
			def czc_fill_subzclass(self,par_oldpath,par_newpath):
					"""
					Fill the new sub-ZClass(es) with the orginal data
					"""
					old_instance = GetOldZClass(self.__extself,par_oldpath)
					new_instance = WriteZClass(self.__extself,par_newpath)
					new_instance.czc_set_basic_iconpath(old_instance.czc_get_basic_iconpath())
					new_instance.czc_set_basic_iconimage(old_instance.czc_get_basic_iconimage())
					new_instance.czc_set_DefinedPermissions(old_instance.czc_get_DefinedPermissions())
					new_instance.czc_set_PermissionMapping(old_instance.czc_get_PermissionMapping())
					new_instance.czc_copy_methods(old_instance.czc_get_oldZClassObj_methods(),old_instance.czc_get_methods(),old_instance.czc_get_oldZClassPath_methods())
					new_instance.czc_set_views(old_instance.czc_get_views())
					for psids in old_instance.czc_get_ps_IdsTitle():
							new_instance.czc_create_ps(psids['id'],psids['title'])
							new_instance.czc_set_ps_Items(psids['id'],old_instance.czc_get_ps_Items(psids['id']))
							new_instance.czc_set_ps_Permissions(psids['id'],old_instance.czc_get_ps_Permissions(psids['id']))		 
					
					
			def czc_set_views(self,par_viewdict):
					"""
					Set views of the new ZClass (view: view)
					"""
					finaldict = []
					for lauf in par_viewdict:
							finaldict.append(lauf)					
					new_viewdict = self.__newZClassObj_views.data()
					
					for alauf in new_viewdict:
							found = 0
							laufnr = 0
							for ilauf in par_viewdict:
									if alauf['label']==ilauf['label']: found=1
									else: laufnr=laufnr+1
							if found==0:
								 finaldict.append(alauf)
							elif found==1:
								 finaldict[laufnr]=alauf
					
					finaltuple = ()
					for lauf in finaldict:
							tmptuple=(lauf,)
							finaltuple = finaltuple + tmptuple 
					
					for lauf in new_viewdict:
							tmp = []
							tmp.append(lauf['label'])
							try:
									self.__newZClassObj_views.manage_delete(tmp)
							except:
									pass
					
					#for lauf in finaldict:
							#self.__newZClassObj_views.manage_add(lauf['label'],lauf['action'],lauf['help'])
					
					self.__newZClassObj_views.setClassAttr('manage_options', finaltuple)				
							
					return finaldict
					
					
			def czc_create_ps(self,par_id,par_title):
					"""
					Create common property sheet
					"""
					o=ZCommonSheet(par_id, par_title)
					self.__newZClassObj_propertysheets._setObject(par_id, o)
					
					
			def czc_set_ps_Items(self,par_id,par_items):
					"""
					Fill common property sheet with data
					"""
					__psobj = self.__extself.unrestrictedTraverse(self.__newZClassPath_propertysheets+'/'+par_id)
					for lauf in par_items:
							#__psobj.manage_addProperty(lauf['id'],'BRK_BW_Funktion',lauf['type'])
							try:
									if lauf['select_variable']!='':
										 __psobj.manage_addProperty(lauf['id'],lauf['select_variable'],lauf['type'])
							except:
									__psobj.manage_addProperty(lauf['id'],lauf['value'],lauf['type'])
							#__psobj.manage_addProperty(lauf['id'],lauf['value'],lauf['type'])
							
							
			def czc_set_ps_Permissions(self,par_Id,par_permclasses):
					"""
					Set permissions of a specific property sheet (par_Id).					 
					"""
					tmp_perm = []
					tmp_classperm = []
					for lauf in par_permclasses:
							tmp_perm.append(lauf['permission_name'])
							tmp_classperm.append(lauf['class_permission'])
					__psobj = self.__extself.unrestrictedTraverse(self.__newZClassPath_propertysheets + '/' + par_Id)
					__psobj.manage_setPermissionMapping(tmp_perm,tmp_classperm)
			
			
			def czc_set_DefinedPermissions(self,par_defperm):
					"""
					Set defined class permissions
					"""					
					self.__newZClassObj.setClassAttr('__ac_permissions__',par_defperm)
					
					
			def czc_set_PermissionMapping(self,par_mapping):
					"""
					Set permission mapping (view: Define Permissions)
					"""
					tmp_perm = []
					tmp_classperm = []
					for lauf in par_mapping:
							tmp_perm.append(lauf['permission_name'])
							tmp_classperm.append(lauf['class_permission'])
					self.__newZClassObj.manage_setPermissionMapping(tmp_perm,tmp_classperm)										 												